#include <iostream>
#include <algorithm>
#include <queue>
#include <cstdio>

using namespace std;

class DFU
{
	int * parents;
	int * sizes;
	int length;


  public:
  	int GetRoot(int id);
	DFU(int len);
	~DFU();
	bool IsConnected(int id1,int id2);
	bool Union(int id1,int id2);
	int GetNumberOfComponents() const;
};

DFU::DFU(int len)
{
	length=len;
	parents=new int[len];
	sizes=new int [len];
	for(int i=0;i<len;i++) {parents[i]=i;sizes[i]=1; }	
}

DFU::~DFU()
{
	delete [] parents;
	delete [] sizes;

}

int DFU::GetRoot(int id)
{
	if(id!=parents[id])
	{
		parents[id]=GetRoot(parents[id]);

	}
	return parents[id];
}

bool DFU::IsConnected(int id1,int id2)
{
	return (GetRoot(id1)==GetRoot(id2));
}


bool DFU::Union(int id1, int id2)
{
	if(IsConnected(id1,id2)) return false;
	cout<<"UNION "<<id1<<" "<<id2<<endl;
	int root1=GetRoot(id1);
	int root2=GetRoot(id2);

	if(sizes[root1]<sizes[root2])
	{
		parents[root1]=root2;
		sizes[root2]+=sizes[root1];
	}
	else
	{
		parents[root2]=root1;
		sizes[root1]+=sizes[root2];
	}
	return true;
}

int DFU::GetNumberOfComponents() const
{
	int num=0;
	for(int i=1;i<length;i++) if(i==parents[i]) num++;
	return num;

}


struct Edge
{
	int x,y,len;
	Edge(int X,int Y,int Len) {x=X;y=Y;len=Len; }
	Edge() {}
	
	bool operator < (const Edge & e) const
	{
		return (!(len<e.len));
	}


};

int Maxs[2005][2005];
int Mins[2005][2005];

int main()
{
	int N,M;
	
	
	
	while(1)
	{
		cin>>N>>M;
		if(!(cin.good())) break;
		
		int x,y,len, sum=0,max_rozdil=0,x2=0,y2=0;
		Edge ed;
		priority_queue<Edge> pq;
		for(int i=0;i<M;i++)
		{
			scanf("%d %d %d",&x,&y,&len);
			pq.push(Edge(x,y,len));
		
		}
		DFU * dfu=new DFU(N+1);
		
		while(!(pq.empty()))
		{
			ed=pq.top();
			pq.pop();
			
			int rx=dfu->GetRoot(ed.x);
			int ry=dfu->GetRoot(ed.y);
			
			if(dfu->Union(ed.x,ed.y))
			{
				Mins[ed.x][ed.y]=Mins[ed.y][ed.x]=ed.len;
				sum+=ed.len;
			}
			else
			{
				Maxs[rx][ry]=Maxs[ry][rx]=max(Maxs[rx][ry], ed.len);
				
				if(max_rozdil<Maxs[rx][ry]-Mins[rx][ry])
				{
					x2=rx;
					y2=ry;
					max_rozdil=Maxs[rx][ry]-Mins[rx][ry];
				}
			
			}
		
		}
		
		//cout<<dfu->GetNumberOfComponents()<<endl;
		
		if(dfu->GetNumberOfComponents()!=1)
		{
			cout<<"disconnected"<<endl;
		}
		else
		{
			cout<<sum - Mins[x2][y2]-Maxs[x2][y2]<<endl;
		
		}

		delete dfu;

	}

	return 0;
}
